@@ -17,6 +17,7 @@ from typing import Union
 
 from filetype import guess_extension as filetype_guess_extension
 
+from .__version__ import __version__
 from .merge import merge_database
 from .tables import journals_table
 from .tables import make_journals_table
@@ -37,8 +38,8 @@ def guess_extension(file: bytes, default: str = "") -> str:
         return ""
     elif (file_type := filetype_guess_extension(file)) is None:
         return default
-    elif (file_ext := str(file_type)) == "zip":
-        return default if default != "zip" else file_ext
+    elif (file_ext := str(file_type)) in (exts := ("zip", "octet-stream")):
+        return default if default not in exts else file_ext
     else:
         return file_ext
 
@@ -60,15 +61,12 @@ class FADatabaseTable:
     def __init__(self, database: 'FADatabase', table: str):
         self.database: 'FADatabase' = database
         self.table: str = table
-        self.columns_info: List[Tuple[str, str]] = [
-            (info[1], info[-1])
-            for info in self.database.connection.execute(f"pragma table_info({self.table})")
-        ]
-        self.columns: List[str] = [name for name, _ in self.columns_info]
-        self.column_id: str = [name for name, pk in self.columns_info if pk == 1][0]
+        self.columns_info_: List[Tuple[str, str]] = []
+        self.columns_: List[str] = []
+        self.column_id_: str = ""
 
     def __len__(self) -> int:
-        return self.database.connection.execute(f"SELECT COUNT({self.column_id}) FROM {self.table}").fetchone()[0]
+        return self.database.connection.execute(f"SELECT COUNT(*) FROM {self.table}").fetchone()[0]
 
     def __getitem__(self, key: Union[Key, Dict[str, Union[List[Value], Value]]]) -> Optional[Dict[str, Value]]:
         key = key if isinstance(key, dict) else {self.column_id: key}
@@ -86,13 +84,56 @@ class FADatabaseTable:
         return self[key] is not None
 
     def __iter__(self) -> Generator[Dict[str, Value], None, None]:
-        entry: Tuple[Value]
-        for entry in self.select():
-            yield {k.upper(): v for k, v in zip(self.columns, entry)}
+        return ({k.upper(): v for k, v in zip(self.columns_, entry)} for entry in self.select())
+
+    @property
+    def columns_info(self) -> List[Tuple[str, str]]:
+        self.columns_info_ = [
+            info[1:]
+            for info in self.database.connection.execute(f"pragma table_info({self.table})")
+        ] if not self.columns_info_ else self.columns_info_
+        return self.columns_info_
+
+    @property
+    def columns(self) -> List[str]:
+        self.columns_ = [name for name, *_ in self.columns_info] if not self.columns_ else self.columns_
+        return self.columns_
+
+    @property
+    def column_id(self) -> str:
+        self.column_id_ = [
+            name
+            for name, *_, pk in self.columns_info if pk == 1
+        ][0]
+        return self.column_id_
+
+    def add_to_list(self, key: Key, values: Dict[str, List[Value]]) -> bool:
+        item: Optional[dict] = self[key]
+        if not item:
+            return False
+        item = {k: item[k] for k in values.keys()}
+        item_new: Dict[str, str] = {k: ",".join(sorted(filter(bool, set(item[k].split(",") + v))))
+                                    for k, v in values.items()}
+        self.update(item_new, key) if item_new != item else None
+        return item_new != item
+
+    def remove_from_list(self, key: Key, values: Dict[str, List[Value]]) -> bool:
+        item: Optional[dict] = self[key]
+        if not item:
+            return False
+        item = {k: item[k] for k in values.keys()}
+        item_new: Dict[str, str] = {k: ",".join(sorted(filter(bool, set(item[k].split(",")) - set(v))))
+                                    for k, v in values.items()}
+        self.update(item_new, key) if item_new != item else None
+        return item_new != item
 
     def reload(self):
         self.__init__(self.database, self.table)
 
+    def cursor_to_dict(self, cursor: Cursor, columns: List[str] = None) -> Generator[Dict[str, Value], None, None]:
+        columns = self.columns if columns is None else columns
+        return ({k.upper(): v for k, v in zip(columns, entry)} for entry in cursor)
+
     def select(self, query: Dict[str, Union[List[Value], Value]] = None, columns: List[str] = None,
                query_and: bool = True, query_and_values: bool = False, like: bool = False,
                order: List[str] = None, limit: int = 0, offset: int = 0
@@ -119,14 +160,23 @@ class FADatabaseTable:
             [v for values in query.values() for v in values]
         )
 
-    def cursor_to_dict(self, cursor: Cursor, columns: List[str] = None) -> Generator[Dict[str, Value], None, None]:
+    def select_sql(self, where: str = "", values: List[Value] = None, columns: List[str] = None,
+                   order: List[str] = None, limit: int = 0, offset: int = 0) -> Cursor:
         columns = self.columns if columns is None else columns
-        return ({k.upper(): v for k, v in zip(columns, entry)} for entry in cursor)
+        order = [] if order is None else order
+        return self.database.connection.execute(
+            f"""SELECT {','.join(columns)} FROM {self.table}
+            {f' WHERE {where} ' if where else ''}
+            {f' ORDER BY {",".join(order)}' if order else ''}
+            {f' LIMIT {limit}' if limit > 0 else ''}
+            {f' OFFSET {offset}' if limit > 0 and offset > 0 else ''}""",
+            [] if values is None else values
+        )
 
     def insert(self, values: Dict[str, Value], replace: bool = True):
         self.database.connection.execute(
             f"""INSERT OR {'REPLACE' if replace else 'IGNORE'} INTO {self.table}
-            ({','.join(col for col in values)}) VALUES ({','.join('?' for _ in values)})""",
+            ({','.join(values.keys())}) VALUES ({','.join(['?'] * len(values))})""",
             [v for v in values.values()]
         )
 
@@ -143,10 +193,20 @@ class FADatabaseTable:
 
 
 class FADatabaseJournals(FADatabaseTable):
-    def save_journal(self, journal: Dict[str, Union[int, str]]):
-        journal = {k.upper(): v for k, v in journal.items()}
+    def save_journal(self, journal: Dict[str, Union[int, str, list]]):
+        journal = {k.upper(): ",".join(map(str, v)) if isinstance(v, list) else v for k, v in journal.items()}
         self[journal["ID"]] = journal
 
+    def add_mention(self, journal_id: int, user: str) -> bool:
+        user = clean_username(user)
+        assert len(user) > 0, "User cannot be empty"
+        return self.add_to_list(journal_id, {"MENTIONS": [user]})
+
+    def remove_mention(self, journal_id: int, user: str) -> bool:
+        user = clean_username(user)
+        assert len(user) > 0, "User cannot be empty"
+        return self.remove_from_list(journal_id, {"MENTIONS": [user]})
+
 
 class FADatabaseSettings(FADatabaseTable):
     def __getitem__(self, setting: str) -> Optional[str]:
@@ -177,8 +237,9 @@ class FADatabaseSubmissions(FADatabaseTable):
 
         return file_ext
 
-    def save_submission(self, submission: Dict[str, Union[int, str]], file: Optional[bytes] = None):
-        submission = {k.upper(): v for k, v in submission.items()}
+    def save_submission(self, submission: Dict[str, Union[int, str, list]], file: Optional[bytes] = None):
+        submission = {k.upper(): ",".join(map(str, v)) if isinstance(v, list) else v for k, v in submission.items()}
+        submission = {k: submission.get(k, "") for k in {*submission.keys(), *self.columns}}
 
         submission["FILEEXT"] = name.split(".")[-1] if "." in (name := submission["FILELINK"].split("/")[-1]) else ""
         submission["FILESAVED"] = bool(file)
@@ -187,12 +248,37 @@ class FADatabaseSubmissions(FADatabaseTable):
 
         self[submission["ID"]] = submission
 
+    def set_folder(self, submission_id: int, folder: str) -> bool:
+        folder_old: str = self[submission_id]["FOLDER"]
+        self.update({"FOLDER": folder}, submission_id) if folder_old != (folder := folder.lower().strip()) else None
+        return folder_old != folder
+
+    def add_favorite(self, submission_id: int, user: str) -> bool:
+        user = clean_username(user)
+        assert len(user) > 0, "User cannot be empty"
+        return self.add_to_list(submission_id, {"FAVORITE": [user]})
+
+    def remove_favorite(self, submission_id: int, user: str) -> bool:
+        user = clean_username(user)
+        assert len(user) > 0, "User cannot be empty"
+        return self.remove_from_list(submission_id, {"FAVORITE": [user]})
+
+    def add_mention(self, submission_id: int, user: str) -> bool:
+        user = clean_username(user)
+        assert len(user) > 0, "User cannot be empty"
+        return self.add_to_list(submission_id, {"MENTIONS": [user]})
+
+    def remove_mention(self, submission_id: int, user: str) -> bool:
+        user = clean_username(user)
+        assert len(user) > 0, "User cannot be empty"
+        return self.remove_from_list(submission_id, {"MENTIONS": [user]})
+
 
 class FADatabaseUsers(FADatabaseTable):
     def new_user(self, user: str):
         user = clean_username(user)
         if user not in self:
-            self[user] = {f: "" for f in self.columns}
+            self[user] = {f: "" for f in self.columns_}
 
     def enable_user(self, user: str):
         user = clean_username(user)
@@ -211,67 +297,13 @@ class FADatabaseUsers(FADatabaseTable):
             for f in filter(bool, user_entry["FOLDERS"].split(","))
         ])}, user)
 
-    def add_item(self, user: str, folder: str, item: str) -> bool:
-        folder = folder.upper()
-        user = clean_username(user)
-        if (user_entry := self[user]) is None:
-            return False
-
-        if item not in (items := user_entry[folder].split(",")):
-            user_entry[folder] = ",".join(sorted([item, *filter(bool, items)]))
-            self.update(user_entry, user)
-            return True
-
-        return False
-
-    def remove_item(self, user: str, folder: str, item: str) -> bool:
-        folder = folder.upper()
-        user = clean_username(user)
-        if (user_entry := self[user]) is None:
-            return False
-
-        if item in (items := user_entry[folder].split(",")):
-            user_entry[folder] = ",".join(filter(lambda i: i != item, items))
-            self.update(user_entry, user)
-            return True
-
-        return False
-
     def add_user_folder(self, user: str, folder: str):
         folder = folder.lower()
-        self.add_item(clean_username(user), "FOLDERS", folder)
+        self.add_to_list(clean_username(user), {"FOLDERS": [folder]})
 
     def remove_user_folder(self, user: str, folder: str):
         folder = folder.lower()
-        self.remove_item(clean_username(user), "FOLDERS", folder)
-
-    def find_from_submission(self, submission_id: int) -> Cursor:
-        return self.select(
-            query=dict(zip(("GALLERY", "SCRAPS", "FAVORITES", "MENTIONS"), [f"%{submission_id:010}%"] * 4)),
-            query_and=False,
-            like=True
-        )
-
-    def find_from_journal(self, journal_id: int) -> Cursor:
-        return self.select({"JOURNALS": f"%{journal_id:010}%"}, like=True)
-
-    def add_submission(self, user: str, folder: str, submission_id: int):
-        folder = folder.upper()
-        self.add_item(clean_username(user), folder, f"{submission_id:010}")
-
-    def add_journal(self, user: str, journal_id: int):
-        self.add_item(clean_username(user), "JOURNALS", f"{journal_id:010}")
-
-    def remove_submission(self, user: str, submission_id: int):
-        user = clean_username(user)
-        submission_id_fmt: str = f"{submission_id:010}"
-        self.remove_item(user, "GALLERY", submission_id_fmt)
-        self.remove_item(user, "SCRAPS", submission_id_fmt)
-        self.remove_item(user, "FAVORITES", submission_id_fmt)
-        self.remove_item(user, "MENTIONS", submission_id_fmt)
-
-    def remove_journal(self, user: str, journal_id: int):
-        self.remove_item(clean_username(user), "JOURNALS", f"{journal_id:010}")
+        self.remove_from_list(clean_username(user), {"FOLDERS": [folder]})
 
 
 class FADatabase:
@@ -298,8 +330,7 @@ class FADatabase:
         return FADatabaseTable(self, table)
 
     def __iter__(self) -> Generator[Tuple[str, FADatabaseTable], None, None]:
-        for table in self.tables:
-            yield table, FADatabaseTable(self, table)
+        return ((table, FADatabaseTable(self, table)) for table in self.tables)
 
     def __contains__(self, table: str) -> bool:
         return table in self.tables
@@ -335,7 +366,7 @@ class FADatabase:
         return self.total_changes == self.committed_changes
 
     def upgrade(self):
-        self.connection = update_database(self.connection)
+        self.connection = update_database(self.connection, __version__)
 
         self.journals.reload()
         self.settings.reload()
