@@ -1,4 +1,5 @@
 from sqlite3 import Connection
+from json import dumps
 
 from .__version__ import __version__
 
@@ -54,6 +55,8 @@ def make_journals_table(db: Connection):
         TITLE TEXT NOT NULL,
         DATE DATE NOT NULL CHECK (DATE==strftime('%Y-%m-%d',DATE)),
         CONTENT TEXT NOT NULL,
+        MENTIONS TEXT NOT NULL,
+        USERUPDATE INT NOT NULL CHECK (USERUPDATE in (0, 1)),
         PRIMARY KEY (ID ASC));""",
     )
 
@@ -68,13 +71,10 @@ def make_settings_table(db: Connection):
         PRIMARY KEY (SETTING ASC));"""
     )
 
-    db.execute("INSERT OR IGNORE INTO SETTINGS (SETTING, SVALUE) VALUES ('USRN', '0');")
-    db.execute("INSERT OR IGNORE INTO SETTINGS (SETTING, SVALUE) VALUES ('SUBN', '0');")
-    db.execute("INSERT OR IGNORE INTO SETTINGS (SETTING, SVALUE) VALUES ('JRNN', '0');")
-    db.execute("INSERT OR IGNORE INTO SETTINGS (SETTING, SVALUE) VALUES ('HISTORY', '[]');")
-    db.execute("INSERT OR IGNORE INTO SETTINGS (SETTING, SVALUE) VALUES ('COOKIES', '{}');")
-    db.execute("INSERT OR IGNORE INTO SETTINGS (SETTING, SVALUE) VALUES ('FILESFOLDER', 'FA.files');")
-    db.execute("INSERT OR IGNORE INTO SETTINGS (SETTING, SVALUE) VALUES ('VERSION', '{0}');".format(__version__))
+    db.execute(f"INSERT OR IGNORE INTO {settings_table} VALUES ('HISTORY', ?);", (dumps([]),))
+    db.execute(f"INSERT OR IGNORE INTO {settings_table} VALUES ('COOKIES', ?);", (dumps({}),))
+    db.execute(f"INSERT OR IGNORE INTO {settings_table} VALUES ('FILESFOLDER', ?);", ("FA.files",))
+    db.execute(f"INSERT OR IGNORE INTO {settings_table} VALUES ('VERSION', ?);", (__version__,))
 
     db.commit()
 
@@ -95,6 +95,10 @@ def make_submissions_table(db: Connection):
         FILELINK TEXT NOT NULL,
         FILEEXT TEXT NOT NULL,
         FILESAVED INT NOT NULL CHECK (FILESAVED in (0, 1)),
+        FAVORITE TEXT NOT NULL,
+        MENTIONS TEXT NOT NULL,
+        FOLDER TEXT NOT NULL CHECK (FOLDER = 'gallery' OR FOLDER = 'scraps'),
+        USERUPDATE INT NOT NULL CHECK (USERUPDATE in (0, 1)),
         PRIMARY KEY (ID ASC));"""
     )
 
@@ -106,11 +110,6 @@ def make_users_table(db: Connection):
         f"""CREATE TABLE IF NOT EXISTS {users_table}
         (USERNAME TEXT UNIQUE NOT NULL CHECK (length(USERNAME) > 0),
         FOLDERS TEXT NOT NULL,
-        GALLERY TEXT NOT NULL,
-        SCRAPS TEXT NOT NULL,
-        FAVORITES TEXT NOT NULL,
-        MENTIONS TEXT NOT NULL,
-        JOURNALS TEXT NOT NULL,
         PRIMARY KEY (USERNAME ASC));"""
     )
 
